@@ -1086,7 +1086,7 @@ NumericVector fe3d(NumericVector cv1, NumericVector cv2, NumericVector cv3, Nume
     dcv2 = cv2[i]-y;
     dcv3 = cv3[i]-z;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
@@ -1106,7 +1106,7 @@ NumericVector fe3dp1(NumericVector cv1, NumericVector cv2, NumericVector cv3, Nu
     dcv2 = cv2[i]-y;
     dcv3 = cv3[i]-z;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
@@ -1126,7 +1126,7 @@ NumericVector fe3dp2(NumericVector cv1, NumericVector cv2, NumericVector cv3, Nu
     if(dcv2 < -p2/2.0) dcv2 += p2;
     dcv3 = cv3[i]-z;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
@@ -1146,7 +1146,7 @@ NumericVector fe3dp3(NumericVector cv1, NumericVector cv2, NumericVector cv3, Nu
     if(dcv3 >  p3/2.0) dcv3 -= p3;
     if(dcv3 < -p3/2.0) dcv3 += p3;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
@@ -1168,7 +1168,7 @@ NumericVector fe3dp12(NumericVector cv1, NumericVector cv2, NumericVector cv3, N
     if(dcv2 < -p2/2.0) dcv2 += p2;
     dcv3 = cv3[i]-z;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
@@ -1190,7 +1190,7 @@ NumericVector fe3dp13(NumericVector cv1, NumericVector cv2, NumericVector cv3, N
     if(dcv3 >  p3/2.0) dcv3 -= p3;
     if(dcv3 < -p3/2.0) dcv3 += p3;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
@@ -1212,7 +1212,7 @@ NumericVector fe3dp23(NumericVector cv1, NumericVector cv2, NumericVector cv3, N
     if(dcv3 >  p3/2.0) dcv3 -= p3;
     if(dcv3 < -p3/2.0) dcv3 += p3;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
@@ -1236,8 +1236,160 @@ NumericVector fe3dp123(NumericVector cv1, NumericVector cv2, NumericVector cv3,
     if(dcv3 >  p3/2.0) dcv3 -= p3;
     if(dcv3 < -p3/2.0) dcv3 += p3;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
 
+// [[Rcpp::export]]
+double f3d(NumericVector cv1, NumericVector cv2, NumericVector cv3, NumericVector width1, NumericVector width2, NumericVector width3, NumericVector heights, double x, double y, double z, int tmin, int tmax) {
+  double dcv1;
+  double dcv2;
+  double dcv3;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    dcv2 = cv2[i]-y;
+    dcv3 = cv3[i]-z;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
+  }
+  return v;
+}
+
+// [[Rcpp::export]]
+double f3dp1(NumericVector cv1, NumericVector cv2, NumericVector cv3, NumericVector width1, NumericVector width2, NumericVector width3, NumericVector heights, double x, double y, double z, double p1, int tmin, int tmax) {
+  double dcv1;
+  double dcv2;
+  double dcv3;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    if(dcv1 >  p1/2.0) dcv1 -= p1;
+    if(dcv1 < -p1/2.0) dcv1 += p1;
+    dcv2 = cv2[i]-y;
+    dcv3 = cv3[i]-z;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
+  }
+  return v;
+}
+
+// [[Rcpp::export]]
+double f3dp2(NumericVector cv1, NumericVector cv2, NumericVector cv3, NumericVector width1, NumericVector width2, NumericVector width3, NumericVector heights, double x, double y, double z, double p2, int tmin, int tmax) {
+  double dcv1;
+  double dcv2;
+  double dcv3;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    dcv2 = cv2[i]-y;
+    if(dcv2 >  p2/2.0) dcv2 -= p2;
+    if(dcv2 < -p2/2.0) dcv2 += p2;
+    dcv3 = cv3[i]-z;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
+  }
+  return v;
+}
+
+// [[Rcpp::export]]
+double f3dp3(NumericVector cv1, NumericVector cv2, NumericVector cv3, NumericVector width1, NumericVector width2, NumericVector width3, NumericVector heights, double x, double y, double z, double p3, int tmin, int tmax) {
+  double dcv1;
+  double dcv2;
+  double dcv3;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    dcv2 = cv2[i]-y;
+    dcv3 = cv3[i]-z;
+    if(dcv3 >  p3/2.0) dcv3 -= p3;
+    if(dcv3 < -p3/2.0) dcv3 += p3;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
+  }
+  return v;
+}
+
+// [[Rcpp::export]]
+double f3dp12(NumericVector cv1, NumericVector cv2, NumericVector cv3, NumericVector width1, NumericVector width2, NumericVector width3, NumericVector heights, double x, double y, double z, double p1, double p2, int tmin, int tmax) {
+  double dcv1;
+  double dcv2;
+  double dcv3;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    if(dcv1 >  p1/2.0) dcv1 -= p1;
+    if(dcv1 < -p1/2.0) dcv1 += p1;
+    dcv2 = cv2[i]-y;
+    if(dcv2 >  p2/2.0) dcv2 -= p2;
+    if(dcv2 < -p2/2.0) dcv2 += p2;
+    dcv3 = cv3[i]-z;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
+  }
+  return v;
+}
+
+// [[Rcpp::export]]
+double f3dp13(NumericVector cv1, NumericVector cv2, NumericVector cv3, NumericVector width1, NumericVector width2, NumericVector width3, NumericVector heights, double x, double y, double z, double p1, double p3, int tmin, int tmax) {
+  double dcv1;
+  double dcv2;
+  double dcv3;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    if(dcv1 >  p1/2.0) dcv1 -= p1;
+    if(dcv1 < -p1/2.0) dcv1 += p1;
+    dcv2 = cv2[i]-y;
+    dcv3 = cv3[i]-z;
+    if(dcv3 >  p3/2.0) dcv3 -= p3;
+    if(dcv3 < -p3/2.0) dcv3 += p3;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
+  }
+  return v;
+}
+
+// [[Rcpp::export]]
+double f3dp23(NumericVector cv1, NumericVector cv2, NumericVector cv3, NumericVector width1, NumericVector width2, NumericVector width3, NumericVector heights, double x, double y, double z, double p2, double p3, int tmin, int tmax) {
+  double dcv1;
+  double dcv2;
+  double dcv3;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    dcv2 = cv2[i]-y;
+    if(dcv2 >  p2/2.0) dcv2 -= p2;
+    if(dcv2 < -p2/2.0) dcv2 += p2;
+    dcv3 = cv3[i]-z;
+    if(dcv3 >  p3/2.0) dcv3 -= p3;
+    if(dcv3 < -p3/2.0) dcv3 += p3;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
+  }
+  return v;
+}
+
+// [[Rcpp::export]]
+double f3dp123(NumericVector cv1, NumericVector cv2, NumericVector cv3, NumericVector width1, NumericVector width2, NumericVector width3, NumericVector heights, double x, double y, double z, double p1, double p2, double p3, int tmin, int tmax) {
+  double dcv1;
+  double dcv2;
+  double dcv3;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    if(dcv1 >  p1/2.0) dcv1 -= p1;
+    if(dcv1 < -p1/2.0) dcv1 += p1;
+    dcv2 = cv2[i]-y;
+    if(dcv2 >  p2/2.0) dcv2 -= p2;
+    if(dcv2 < -p2/2.0) dcv2 += p2;
+    dcv3 = cv3[i]-z;
+    if(dcv3 >  p3/2.0) dcv3 -= p3;
+    if(dcv3 < -p3/2.0) dcv3 += p3;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]-dcv3*dcv3/2.0/width3[i]/width3[i]);
+  }
+  return v;
+}
+
